/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <tizen.h>
#include <widget_app.h>
#include <dlog.h>
#include "log.h"
#include "view.h"
#include "data.h"

/*
 * This function handle click event.
 */
static void clicked_cb(void *user_data)
{
	/**
	 *  You can write your code here for click event handling.
	 */
	dlog_print(DLOG_DEBUG, LOG_TAG, "clicked a widget.");
}

/*
 * Create widget instance
 */
static int widget_instance_create(widget_context_h context, bundle *content, int w, int h, void *user_data)
{
	void *wid = NULL;
	char *news_text = NULL;
	char *more_text = NULL;

	if (content != NULL) {
		/* Recover the previous status with the bundle object. */
	}

	/* Create main view */
	wid = view_create(context, w, h);
	if (wid == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a view.");
		return WIDGET_ERROR_FAULT;
	}

	/* Set image to "widget.bg" part of EDC */
	view_set_image(wid, "widget.bg", "images/bg.png");

	/* Set text to "widget.text" part of EDC */
	news_text = data_get_text("widget.text");
	view_set_text(wid, "widget.text", news_text);
	free(news_text);

	/* Set text to "widget.more" part of EDC */
	more_text = data_get_text("widget.more");
	view_set_text(wid, "widget.more", more_text);
	free(more_text);

	/* Set callback event handler */
	view_set_event_callback(wid, "widget.event", WIDGET_EVENT_CLICKED, (void *) clicked_cb);

	widget_app_context_set_tag(context, wid);

	return WIDGET_ERROR_NONE;
}

/*
 * Destroy widget instance
 */
static int widget_instance_destroy(widget_context_h context, widget_app_destroy_type_e reason, bundle *content_info, void *user_data)
{
	void *wid = NULL;
	widget_app_context_get_tag(context, (void**) &wid);

	view_destroy(wid);

	return WIDGET_ERROR_NONE;
}

/*
 * This function handle widget instance pause event
 */
static int widget_instance_pause(widget_context_h context, void *user_data)
{
	/* Take necessary actions when widget instance becomes invisible. */
	return WIDGET_ERROR_NONE;
}

/*
 * This function handle widget instance resume event
 */
static int widget_instance_resume(widget_context_h context, void *user_data)
{
	/* Take necessary actions when widget instance becomes visible. */
	return WIDGET_ERROR_NONE;
}

/*
 * This function handle widget instance update event
 */
static int widget_instance_update(widget_context_h context, bundle *content, int force, void *user_data)
{
	/* Take necessary actions when widget instance should be updated. */
	return WIDGET_ERROR_NONE;
}

/*
 * This function handle widget instance resize event
 */
static int widget_instance_resize(widget_context_h context, int w, int h, void *user_data)
{
	void *wid = NULL;

	widget_app_context_get_tag(context, (void**) &wid);
	view_resize(wid, w, h);

	return WIDGET_ERROR_NONE;
}

/*
 * This function handle widget instance language change event
 */
static void widget_app_lang_changed(app_event_info_h event_info, void *user_data)
{
	/* APP_EVENT_LANGUAGE_CHANGED */
	char *locale = NULL;
	app_event_get_language(event_info, &locale);
	elm_language_set(locale);
	free(locale);
}

/*
 * This function handle widget instance region change event
 */
static void widget_app_region_changed(app_event_info_h event_info, void *user_data)
{
	/* APP_EVENT_REGION_FORMAT_CHANGED */
}

/*
 * This function handle widget create event
 */
static widget_class_h widget_app_create(void *user_data)
{
	app_event_handler_h handlers[5] = {NULL, };

	widget_app_add_event_handler(&handlers[APP_EVENT_LANGUAGE_CHANGED],
			APP_EVENT_LANGUAGE_CHANGED, widget_app_lang_changed, user_data);
	widget_app_add_event_handler(&handlers[APP_EVENT_REGION_FORMAT_CHANGED],
			APP_EVENT_REGION_FORMAT_CHANGED, widget_app_region_changed,
			user_data);

	widget_instance_lifecycle_callback_s ops = {
			.create = widget_instance_create,
			.destroy = widget_instance_destroy,
			.pause = widget_instance_pause,
			.resume = widget_instance_resume,
			.update = widget_instance_update,
			.resize = widget_instance_resize,
	};

	return widget_app_class_create(ops, user_data);
}

/*
 * This function handle widget terminate event
 */
static void widget_app_terminate(void *user_data)
{
	/* Release all resources. */
}

/*
 * The main function of the app
 */
int main(int argc, char *argv[])
{
	widget_app_lifecycle_callback_s ops = {0, };
	int ret;

	ops.create = widget_app_create;
	ops.terminate = widget_app_terminate;

	ret = widget_app_main(argc, argv, &ops, NULL);
	if (ret != WIDGET_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "widget_app_main() is failed. err = %d", ret);
	}

	return ret;
}
